﻿using ToolsDesktop.Interfaces;
using System;
using System.Drawing;
using System.Windows.Forms;

namespace ToolsDesktop.BaseControls
{
	/// <summary>
	/// Контрол, предоставляющий работу с перечислениями
	/// </summary>
	public partial class BaseControlEnum : AbstractBaseControl, IBaseControl
	{
		/// <summary>
		/// Конструктор
		/// </summary>
		/// <param name="propertyName"></param>
		/// <param name="mustFilling"></param>
		/// <param name="readOnly"></param>
		/// <param name="enumType"></param>
		public BaseControlEnum(string propertyName, bool mustFilling, bool readOnly, Type enumType) : base(propertyName, mustFilling, readOnly)
		{
			InitializeComponent();
			_baseControl = this;

			if (enumType.Name.StartsWith("Nullable"))
			{
				enumType = Nullable.GetUnderlyingType(enumType);
			}

			comboBox.Items.Clear();
			foreach (var val in Enum.GetValues(enumType))
			{
				comboBox.Items.Add(val);
			}
			comboBox.SelectedIndexChanged += (object sender, EventArgs e) => { CallOnValueChangeEvent(); };
			panelControl.Controls.Add(comboBox);

			if (!_mustFilling)
			{
				checkBoxNullable.CheckedChanged += (object sender, EventArgs e) =>
				{
					comboBox.Enabled = !(sender as CheckBox).Checked;
					CallOnValueChangeEvent();
				};
				panelControl.Controls.Add(checkBoxNullable);
			}
		}

		public void SetDefaultValue() => _originalValue = null;

		public void SetValueToControl(object value)
		{
			if (value != null)
			{
				comboBox.SelectedIndex = comboBox.Items.IndexOf(value);
			}
			else if (!_mustFilling)
			{
				checkBoxNullable.Checked = true;
			}
		}

		public void DropValueForControl()
		{
			if (_originalValue != null)
			{
				comboBox.SelectedIndex = comboBox.Items.IndexOf(_originalValue);
			}
			else if (!_mustFilling)
			{
				checkBoxNullable.Checked = true;
			}
		}

		public bool CheckValueForControl()
		{
			if (_mustFilling && comboBox.SelectedIndex == -1)
			{
				BackColor = Color.OrangeRed;
				return false;
			}
			return true;
		}

		public object GetValueFromControl()
		{
			if (_mustFilling && comboBox.SelectedIndex == -1)
			{
				throw new ArgumentNullException($"Поле свойства '{labelTitle.Text}' должно быть заполнено");
			}
			return checkBoxNullable.Checked ? null : comboBox.SelectedItem;
		}

		public string GetPropertyName() => _propertyName;
	}
}
