﻿using System;

namespace ToolsModule.Attributes
{
	/// <summary>
	/// Оописание класса из базы данных, его назначение (требуется для выстраивания иерархии с описанием классов, 
	/// применяется к классам-описывающим сущности в хранилище)
	/// </summary>
	[AttributeUsage(AttributeTargets.Class | AttributeTargets.Interface)]
	public class EntityDescriptionAttribute : Attribute
	{
		/// <summary>
		/// Название сущности, которую описывает класс
		/// </summary>
		public string EntityName { get; set; }

		/// <summary>
		/// Описание назначения сущности
		/// </summary>
		public string Description { get; set; }

		/// <summary>
		/// Описание класса из базы данных, его назначение
		/// </summary>
		/// <param name="entityName">Название сущности, которую описывает класс</param>
		/// <param name="description">Описание назначения сущности</param>
		public EntityDescriptionAttribute(string entityName, string description)
        {
			EntityName = entityName;
			Description = description;
        }
    }
}