﻿using System;

namespace ToolsModule.Attributes
{
	/// <summary>
	/// Настройка отображения свойства класса при табличном выводе списка объектов класса (требуется для автоматизации вывода списка, 
	/// применяется к классам ElementViewModel)
	/// </summary>
	[AttributeUsage(AttributeTargets.Property)]
	public class ViewModelControlListPropertyAttribute : Attribute
	{
		/// <summary>
		/// Название на форме
		/// </summary>
		public string DisplayName { get; set; }

		/// <summary>
		/// Скрывать или нет при выводе списка
		/// </summary>
		public bool IsHide { get; set; } = false;

		/// <summary>
		/// Ширина колонки
		/// </summary>
		public int ColumnWidth { get; set; } = 0;

		/// <summary>
		/// Формат вывода в колонке
		/// </summary>
		public string DefaultCellStyleFormat { get; set; } = null;

		/// <summary>
		/// Настройка отображения элемента в контролах
		/// </summary>
		/// <param name="displayName">Название на форме</param>
		public ViewModelControlListPropertyAttribute(string displayName)
		{
			DisplayName = displayName;
		}
	}
}