﻿using CoreModels.ModelsDepartment;
using ToolsModule.Attributes;
using ToolsModule.Enums;
using ToolsModule.ViewModels;
using System;

namespace DepartmentBusinessLogic.ViewModels
{
	/// <summary>
	/// Список преподавателей
	/// </summary>
	public class LecturerListViewModel : ListViewModel<LecturerViewModel> { }

	/// <summary>
	/// Элемент преподаватель
	/// </summary>
	[ViewModelControlElementClass(HaveDependenceEntities = true, Width = 800, Height = 700)]
	[ViewModelControlElementDependenceEntity(Title = "Должности", Order = 1, ParentPropertyName = "LecturerId",
		ControlTypeObject = "DepartmentWindowsDesktop.EntityControls.ControlLecturerPostList, DepartmentWindowsDesktop")]
	[ViewModelControlElementDependenceEntity(Title = "Напрвления обучения", Order = 2, ParentPropertyName = "LecturerId",
		ControlTypeObject = "DepartmentWindowsDesktop.EntityControls.ControlEducationDirectionList, DepartmentWindowsDesktop")]
	[ViewModelControlElementDependenceEntity(Title = "Кураторство групп", Order = 3, ParentPropertyName = "LecturerId",
		ControlTypeObject = "DepartmentWindowsDesktop.EntityControls.ControlStudentGroupList, DepartmentWindowsDesktop")]
	public class LecturerViewModel : ElementViewModel, ILecturerModel
	{
		[ViewModelControlElementProperty("Пользователь", ControlType.ControlGuid, MustHaveValue = true, ReadOnly = false, ControlTypeObject = "SecurityWindowsDesktop.EntityControls.ControlUserList, SecurityWindowsDesktop")]
		public Guid UserId { get; set; }

		[ViewModelControlElementProperty("Уч. степень", ControlType.ControlGuid, MustHaveValue = false, ReadOnly = false, ControlTypeObject = "DepartmentWindowsDesktop.EntityControls.ControlLecturerAcademicDegreeList, DepartmentWindowsDesktop")]
		public Guid? LecturerAcademicDegreeId { get; set; }

		[ViewModelControlListProperty("Уч. степень")]
		[MapConfiguration("LecturerAcademicDegree.LecturerAcademicDegreeName", IsDifficle = true)]
		public string LecturerAcademicDegree { get; set; }

		[ViewModelControlElementProperty("Уч. звание", ControlType.ControlGuid, MustHaveValue = false, ReadOnly = false, ControlTypeObject = "DepartmentWindowsDesktop.EntityControls.ControlLecturerAcademicRankList, DepartmentWindowsDesktop")]
		public Guid? LecturerAcademicRankId { get; set; }

		[ViewModelControlListProperty("Уч. звание")]
		[MapConfiguration("LecturerAcademicRank.LecturerAcademicRankName", IsDifficle = true)]
		public string LecturerAcademicRank { get; set; }

		[ViewModelControlListProperty("Фамилия")]
		[ViewModelControlElementProperty("Фамилия", ControlType.ControlString, MustHaveValue = true)]
		public string LastName { get; set; }

		[ViewModelControlListProperty("Имя")]
		[ViewModelControlElementProperty("Имя", ControlType.ControlString, MustHaveValue = true)]
		public string FirstName { get; set; }

		[ViewModelControlListProperty("Отчество")]
		[ViewModelControlElementProperty("Отчество", ControlType.ControlString, MustHaveValue = true)]
		public string Patronymic { get; set; }

		[ViewModelControlElementProperty("Аббревиатура", ControlType.ControlString)]
		public string Abbreviation { get; set; }

		[ViewModelControlListProperty("Дата рожд.", ColumnWidth = 100, DefaultCellStyleFormat = "dd.MM.yyyy")]
		[ViewModelControlElementProperty("Дата рожд.", ControlType.ControlDateTime, MustHaveValue = true)]
		public DateTime DateBirth { get; set; }

		[ViewModelControlListProperty("Адрес", ColumnWidth = 90)]
		[ViewModelControlElementProperty("Адрес", ControlType.ControlString, MustHaveValue = true)]
		public string Address { get; set; }

		[ViewModelControlListProperty("Эл. почта", ColumnWidth = 90)]
		[ViewModelControlElementProperty("Эл. почта", ControlType.ControlString, MustHaveValue = true)]
		public string Email { get; set; }

		[ViewModelControlListProperty("Моб. номер", ColumnWidth = 100)]
		[ViewModelControlElementProperty("Моб. номер", ControlType.ControlString, MustHaveValue = true)]
		public string MobileNumber { get; set; }

		[ViewModelControlListProperty("Дом. номер", ColumnWidth = 100)]
		[ViewModelControlElementProperty("Дом. номер", ControlType.ControlString)]
		public string HomeNumber { get; set; }

		[ViewModelControlElementProperty("Описание", ControlType.ControlText)]
		public string Description { get; set; }

		[ViewModelControlElementProperty("Фото", ControlType.ControlImage, Width = 200, Height = 200)]
		public byte[] Photo { get; set; }

		[ViewModelControlElementProperty("Для внутр. пользования", ControlType.ControlBool, MustHaveValue = true)]
		public bool OnlyForPrivate { get; set; }

		[ViewModelControlListProperty("Гр. эл.без.", ColumnWidth = 90)]
		[ViewModelControlElementProperty("Группа эл.безоп", ControlType.ControlString, MustHaveValue = true)]
		public string GroupElectricalSafety { get; set; }
	}
}