﻿using CoreModels.Enums.Department;
using CoreModels.ModelsDepartment;
using ToolsModule.Attributes;
using ToolsModule.Enums;
using ToolsModule.ViewModels;
using System;

namespace DepartmentBusinessLogic.ViewModels
{
    /// <summary>
    /// Список студентов
    /// </summary>
    public class StudentListViewModel : ListViewModel<StudentViewModel> { }

	/// <summary>
	/// Элемент студент
	/// </summary>
	[ViewModelControlElementClass(HaveDependenceEntities = true, Width = 800, Height = 500)]
	[ViewModelControlElementDependenceEntity(Title = "Приказы по студенту", Order = 1, ParentPropertyName = "StudentId",
		ControlTypeObject = "DepartmentWindowsDesktop.EntityControls.ControlOrderStudentRecordList, DepartmentWindowsDesktop")]
	public class StudentViewModel : ElementViewModel, IStudentModel
	{
		[ViewModelControlElementProperty("Пользователь", ControlType.ControlGuid, MustHaveValue = true, ReadOnly = false, ControlTypeObject = "SecurityWindowsDesktop.EntityControls.ControlUserList, SecurityWindowsDesktop")]
		public Guid UserId { get; set; }

		[ViewModelControlElementProperty("Группа", ControlType.ControlGuid, MustHaveValue = false, ReadOnly = false, ControlTypeObject = "DepartmentWindowsDesktop.EntityControls.ControlStudentGroupList, DepartmentWindowsDesktop")]
		public Guid? StudentGroupId { get; set; }

		[MapConfiguration("StudentGroup.ToString", IsDifficle = true)]
		public string StudentGroupName { get; set; }

		[ViewModelControlElementProperty("Идентификатор универа", ControlType.ControlString, MustHaveValue = true, ReadOnly = true)]
		public string Iduniv { get; set; }

		[ViewModelControlListProperty("Номер зачетки")]
		[ViewModelControlElementProperty("Номер зачетки", ControlType.ControlString, MustHaveValue = true, ReadOnly = true)]
		public string NumberOfBook { get; set; }

		[ViewModelControlListProperty("Фамилия")]
		[ViewModelControlElementProperty("Фамилия", ControlType.ControlString, MustHaveValue = true)]
		public string LastName { get; set; }

		[ViewModelControlListProperty("Имя")]
		[ViewModelControlElementProperty("Имя", ControlType.ControlString, MustHaveValue = true)]
		public string FirstName { get; set; }

		[ViewModelControlListProperty("Отчество")]
		[ViewModelControlElementProperty("Отчество", ControlType.ControlString, MustHaveValue = true)]
		public string Patronymic { get; set; }

		[ViewModelControlListProperty("Эл. почта", ColumnWidth = 90)]
		[ViewModelControlElementProperty("Эл. почта", ControlType.ControlString, MustHaveValue = true)]
		public string Email { get; set; }

		[ViewModelControlElementProperty("Описание", ControlType.ControlText)]
		public string Description { get; set; }

		[ViewModelControlElementProperty("Статус", ControlType.ControlEnum, MustHaveValue = true)]
		public StudentState StudentState { get; set; }

		[ViewModelControlListProperty("Статус", ColumnWidth = 90)]
		public string StudentStateTitle => StudentState.ToString("G");

		[ViewModelControlElementProperty("Фото", ControlType.ControlImage, Width = 200, Height = 200)]
		public byte[] Photo { get; set; }

		[ViewModelControlElementProperty("Староста", ControlType.ControlBool, MustHaveValue = true)]
		public bool IsSteward { get; set; }
	}
}