﻿using DepartmentWindowsDesktop.EntityControls;
using ToolsDesktop.Interfaces;
using ToolsDesktop.Models;
using ToolsModule.BindingModels;
using ToolsModule.BusinessLogics;
using ToolsModule.Enums;
using ToolsModule.Interfaces;
using ToolsModule.Models;
using System.Collections.Generic;

namespace DepartmentWindowsDesktop
{
	public class DepartmentWindowDesktopExtension : IWindowDesktopExtension
	{
		public List<WindowDesktopExtensionControlModel> GetListControlEntityList()
		{
			var manager = DependencyManager.Instance.Resolve<ISecurityManager>();
			if (manager == null)
			{
				return null;
			}

			if (!manager.CheckAccess(new SecurityManagerCheckAccessModel(new AccessBindingModel { UserIdForAccess = manager.User },
				AccessOperation.Кафедра, AccessType.View, "Кафедра")))
			{
				return null;
			}

			var list = new List<WindowDesktopExtensionControlModel>
			{
				new WindowDesktopExtensionControlModel { Title = "Кафедра" }
			};
			List<IControl> _controls = new()
			{
				new ControlPostList(),
				new ControlEmployeeList(),
				new ControlClassroomList(),
				new ControlDisciplineBlockList(),
				new ControlDisciplineList(),
				new ControlLecturerAcademicDegreeList(),
				new ControlLecturerAcademicRankList(),
				new ControlLecturerList(),
				new ControlEducationDirectionList(),
				new ControlTimeNormList(),
				new ControlAcademicPlanList(),
				new ControlStudentGroupList(),
				new ControlStudentList(),
				new ControlOrderList(),
				new ControlOrderSyncHistoryList()
			};

			foreach (var cntrl in _controls)
			{
				if (manager.CheckAccess(new SecurityManagerCheckAccessModel(new AccessBindingModel { UserIdForAccess = manager.User },
										cntrl.AccessOperation, AccessType.View, cntrl.Title)))
				{
					list.Add(new WindowDesktopExtensionControlModel
					{
						Id = cntrl.ControlId,
						Title = cntrl.Title,
						Control = cntrl
					});
				}
			}

			return list;
		}

		public List<WindowDesktopExtensionControlModel> GetListControlSpecialList()
		{
			return null;
		}
	}
}