﻿using DepartmentBusinessLogic.BindingModels;
using DepartmentBusinessLogic.BusinessLogics;
using DepartmentBusinessLogic.ViewModels;
using ToolsDesktop.Controls;
using ToolsDesktop.Interfaces;
using ToolsDesktop.Models;
using ToolsModule.Enums;
using System;
using System.Threading.Tasks;
using ToolsDesktop.Helpers;
using ToolsModule.BusinessLogics;

namespace DepartmentWindowsDesktop.EntityControls
{
	/// <summary>
	/// Реализация контрола для списка записей приказов по студентам
	/// </summary>
	public partial class ControlOrderStudentRecordList :
		GenericControlEntityList<OrderStudentRecordGetBindingModel, OrderStudentRecordSetBindingModel, OrderStudentRecordListViewModel, OrderStudentRecordViewModel, OrderStudentRecordBusinessLogic>,
		IGenericControlEntityList
	{
		public ControlOrderStudentRecordList()
		{
			InitializeComponent();
			Title = "Приказы";
			ControlId = new Guid("1aa8cf1f-96b0-47aa-bd20-d1563d840534");
			AccessOperation = AccessOperation.Приказы;
			ControlViewEntityElement = new ControlOrderStudentRecordElement();
			_genericControlViewEntityList = this;
		}

		public IControl GetInstanceGenericControl() => new ControlOrderStudentRecordList() { ControlId = Guid.NewGuid() };

		public ControlViewEntityListConfiguration GetConfigControl() => new()
		{
			PaginationOn = false,
			ControlOnMoveElem = new()
			{
				{ "ToolStripMenuItemSyncOrders", ("Синхронизировать приказы студента", async (object sender, EventArgs e) => { await SyncOrders(); }) }
			}
		};

		/// <summary>
		/// Синхронизация приказов
		/// </summary>
		/// <returns></returns>
		private async Task SyncOrders()
		{
			if (!ParentId.HasValue)
            {
				return;
            }
			var orderSyncHistoryBusinessLogic = DependencyManager.Instance.Resolve<OrderSyncHistoryBusinessLogic>();
			var flag = await orderSyncHistoryBusinessLogic.SyncStudentOrders(ParentId.Value);
			if (!flag)
			{
				DialogHelper.MessageException(_businessLogic.Errors, "Ошибки при синхронизации");
			}
			else
			{
				DialogHelper.MessageInformation("Завершено успешно", "Синхронизация приказов");
			}
		}
	}
}